import os
import util
import shutil

import random

randgen = random.Random()

def create_proto(vectype, numvecs, numstates, setname, name="proto"):
    print "Creating %s" % name
    vecmultiplicator = 1
    if "_A" in vectype:
        vecmultiplicator += 1
    if "_D" in vectype:
        vecmultiplicator += 1

    vecsize = numvecs * vecmultiplicator
    output = """~o <VecSize> %d <%s>
~h "%s"
<BeginHMM>
    <NumStates> %d
""" % (vecsize, vectype, name, numstates)
    for x in range(2,numstates):
        output += "    <State> %d\n" % x
        output += "        <Mean> %d\n" % vecsize
        output += "          %s\n" % " ".join(["0.0"]*vecsize)
        output += "        <Variance> %d\n" % vecsize
        output += "          %s\n" % " ".join(["1.0"]*vecsize)
    output += "    <TransP> %d\n" % numstates
    output += "     0.0 1.0 %s\n" % " ".join(["0.0"]*(numstates-2))
    for x in range(1,numstates-2):
        output += "     %s 0.6 0.4 %s\n" % (" ".join(["0.0"]*x),
                                     " ".join(["0.0"]*(numstates-2-x)))
        if x == numstates-3:
            output += "     %s 0.7 0.3\n" % " ".join(["0.0"]*(numstates-2))
    output += "     %s\n" % " ".join(["0.0"]*numstates)
    output += "<EndHMM>"


    outfile = os.path.join("tmp", name)
    fo = open(outfile, 'w')
    fo.write(output)
    return outfile

def create_hed(outfile, numstates, to_mix):
    hed_fo = open(os.path.join("tmp", outfile), 'w')
    hed_fo.write("MU %d {*.state[%d-%d].mix}" % (to_mix, 2, (numstates-1)))

def create_copy_filelist(filelist, setname):
    fo = open(filelist, 'r')
    hcopyfile = os.path.join("tmp", "%s_hcopy.scp" % setname)
    featurefile = os.path.join("tmp", "%s_features.scp" % setname)
    lines = fo.readlines()
    newlines = []
    featurelines = []
    for line in lines:
        line = line.strip()
        newline = line.replace("databaser/", "")
        newline = os.path.join("features", setname, newline)
        newline = "%s.mfc" % newline[:-4]
        util.make_dir(os.path.dirname(newline))
        featurelines.append("%s\n" % newline)
        newline = "%s %s\n" % (line, newline)
        newlines.append(newline)
    fo = open(hcopyfile, 'w')
    fo.writelines(newlines)
    fo = open(featurefile, 'w')
    fo.writelines(featurelines)
    return (hcopyfile, featurefile)

def create_feature_mlf(sourcemlf, setname):
    fo = open(sourcemlf, 'r')
    mlflines = fo.readlines()
    outlines = []
    for line in mlflines:
        if line.startswith('"'):
            line = line.strip().replace("databaser/", "").replace('"', '')
            line = os.path.join("features", setname, line)
            line = '"%s.lab"\n' % line[:-4]
        outlines.append(line)
    outfile = os.path.join("tmp", "%s.mlf" % setname)
    fo = open(outfile, 'w')
    fo.writelines(outlines)
    return outfile

def combine_mlfs(sets, mlf):
    endmlf = []
    first = True
    for set in sets:
        fo = open(create_feature_mlf(mlf, set), 'r')
        if first:
            endmlf.extend(fo.readlines())
            first = False
        else:
            endmlf.extend(fo.readlines()[1:])
    outfile = os.path.join("tmp", "%i.mlf" % randgen.randint(1,100))
    fo = open(outfile, 'w')
    fo.writelines(endmlf)
    return outfile

def combine_features(sets):
    featurelines = []
    for set in sets:
        featurefile = os.path.join("tmp", "%s_features.scp" % set)
        fo = open(featurefile, 'r')
        featurelines.extend(fo.readlines())
    outfile = os.path.join("tmp", "%i.scp" % randgen.randint(100,200))
    fo = open(outfile, 'w')
    fo.writelines(featurelines)
    return outfile


def create_hmmdefs(proto_files, folder):
    util.make_dir(folder)
    fo = open(os.path.join(folder, "hmmdefs"), 'w')
    first = True
    for model, proto_file in proto_files.iteritems():
        lines = open(proto_file, "r").readlines()
        if first:
            skip = False
            first = False
        for line in lines:
            if line.startswith("~h"):
                skip = False
            if skip:
                continue
            fo.write(line)

def create_macros(vectype, numvecs, folder):
    vecmultiplicator = 1
    if "_A" in vectype:
        vecmultiplicator += 1
    if "_D" in vectype:
        vecmultiplicator += 1

    vecsize = numvecs * vecmultiplicator
    outlines = ["~o\n", "  <VecSize> %d\n" % vecsize, "  <%s>\n" % vectype]
    fo = open(os.path.join(folder, "vFloors"), "r")
    vfloorslines = fo.readlines()
    outlines.extend(vfloorslines)
    fo = open(os.path.join(folder, "macros"), "w")
    fo.writelines(outlines)

def fix_sp(sourcefolder, targetfolder):
    util.make_dir(targetfolder)
    shutil.copy(os.path.join(sourcefolder, "macros"),
            os.path.join(targetfolder, "macros"))
    fo = open(os.path.join(sourcefolder, "hmmdefs"), 'r')
    origlines = fo.readlines()
    fo = open(os.path.join(targetfolder, "hmmdefs"), 'w')
    splines = ['~h "sp"\n', "<BEGINHMM>\n", "<NUMSTATES> 3\n",
            "<STATE> 2\n"]
    capture = 0
    for line in origlines:
        if capture > 2 and line.startswith("<STATE>"):
            capture = 0
        if capture >= 2:
            splines.append(line)
            capture += 1
        if line.startswith('~h "sil"'):
            capture = 1
        if capture == 1 and line.startswith("<STATE> 3"):
            capture = 2
    splines.append("<TRANSP> 3\n")
    splines.append(" 0.0 1.0 0.0\n")
    splines.append(" 0.0 0.6 0.4\n")
    splines.append(" 0.0 0.0 0.0\n")
    splines.append("<ENDHMM>\n")
    origlines.extend(splines)
    fo.writelines(origlines)

def mixupdate_hed(modellist, tomix):
    outlines = []
    outlines.append("MU %d {sp.state[2].mix}\n" % (tomix))
    for model, numstates in modellist.iteritems():
        outlines.append("MU %d {%s.state[2-%d].mix}\n" % (tomix, model, numstates-1))
    outfile = os.path.join("tmp", "to_%d_mixtures.hed" % tomix)
    fo = open(outfile, 'w')
    fo.writelines(outlines)
    return outfile

def create_config(setup, train=False):
    if train:
        f = os.path.join("tmp", "%s_train.cfg" % setup['name'])
    else:
        f = os.path.join("tmp", "%s.cfg" % setup['name'])
    fo = open(f, 'w')
    if train:
        keys = ['targetkind',]
    else:
        keys = [
                'targetkind',
                'targetrate',
                'windowsize',
                'usehamming',
                'preemcoef',
                'numchans',
                'ceplifter',
                'numceps',
                'sourcerate',
                'sourcekind',
                'sourceformat',
                'mixtures',
                'warplcutoff',
                'warpucutoff',
                ]
        if setup.get('warpfreq') is not None:
            fo.write("%s=%s\n" % ('WARPFREQ', setup.get('warpfreq')))
        elif setup.get('warpfreqs') is not None:
            key = "WARPFREQ="
            for k, v in setup.get('warpfreqs').items():
                key += "%s:%f;" % (k, v)
            fo.write(key+"\n")
    for key in keys:
        if setup.get(key) is not None:
            fo.write("%s=%s\n" % (key.upper(), setup.get(key)))
    return f
