import filecreation
import trainer
import sets
import os

import logging

logging.basicConfig(filename="lgg.log",level=logging.DEBUG,
        format="%(asctime)-15s %(message)s")

cores = 6

mixture_jump = 2
num_mixtures = 16
mix_path = [1, 3, 6, 9, 12, 16]
runs = 8

for runset in sets.setlist.get_sets():
    if runset['multiset']:
        sets = runset['sets']
    # file creation
    config = filecreation.create_config(runset)
    trainconfig = filecreation.create_config(runset, True)
    if runset['multiset']:
        featurefile = filecreation.combine_features(sets)
    else:
        hcopyfile, featurefile = filecreation.create_copy_filelist(
                runset['filelist'], runset['name'])
    proto_files = {}
    for model, numstates in runset['models'].iteritems():
        proto_files[model] = filecreation.create_proto(
                runset['vectype'], runset['numvecs'],
                numstates, runset['name'], model
                )
    if runset['multiset']:
        mlffile = filecreation.combine_mlfs(sets, runset['mlf'])
    else:
        mlffile = filecreation.create_feature_mlf(runset['mlf'], runset['name'])
    fo = open(runset['modellist'], 'r')
    modellist = fo.readlines()
    for i, line in enumerate(modellist):
        modellist[i] = line.strip()
    spmodellist = list(modellist)
    spmodellist.append("sp")
    # Running htk
    if runset['mwf']:
        trainer.feature_extraction(config, hcopyfile, cores, bilin=runset['bilin'], mwf=runset['mwf'], mlf=runset['copy_mlf'])
    else:
        trainer.feature_extraction(config, hcopyfile, cores, runset['bilin'])
    i = 0
    for model, proto_file in proto_files.iteritems():
        if runset['hard']:
            if i == 0:
                trainer.init_proto(trainconfig, featurefile,
                        os.path.join("models", runset['name'], "hmm1_1"),
                        proto_file, runset['mlf'])
            trainer.hard_init_proto(trainconfig, featurefile,
                    os.path.join("models", runset['name'], "hmm0_1"),
                    proto_file, mlffile, model)
            trainer.hard_reestimation(trainconfig,
                    os.path.join("models", runset['name'], "hmm1_1"),
                    os.path.join("models", runset['name'], "hmm0_1"),
                    featurefile, mlffile, model)
            i += 1
        else:
            trainer.init_proto(trainconfig, featurefile,
                    os.path.join("models", runset['name'], "hmm0_1"),
                    proto_file, runset['mlf'])
    filecreation.create_hmmdefs(
            proto_files,
            os.path.join("models", runset['name'], "hmm0_1")
            )
    filecreation.create_macros(
            runset['vectype'],
            runset['numvecs'],
            os.path.join("models", runset['name'], "hmm0_1")
            )
    for i in range(1,4):
        trainer.reestimation(trainconfig,
                mlffile,
                featurefile,
                os.path.join("models", runset['name'], "hmm%d_1" % (i-1)),
                os.path.join("models", runset['name'], "hmm%d_1" % i),
                runset['modellist'], False, cores
                )
    filecreation.fix_sp(
                os.path.join("models", runset['name'], "hmm3_1"),
                os.path.join("models", runset['name'], "hmm4_1")
                )
    trainer.remix(
                os.path.join("models", runset['name'], "hmm4_1"),
                os.path.join("models", runset['name'], "hmm5_1"),
                "lib/sil.hed",
                runset['spmodellist'])
    if runset['multiset']:
        spmlffile = filecreation.combine_mlfs(sets, runset['spmlf'])
    else:
        spmlffile = filecreation.create_feature_mlf(runset['spmlf'], runset['name'])
    for i in range(6,9):
        trainer.reestimation(trainconfig,
                spmlffile,
                featurefile,
                os.path.join("models", runset['name'], "hmm%d_1" % (i-1)),
                os.path.join("models", runset['name'], "hmm%d_1" % i),
                runset['spmodellist'], True, cores
                )
    for i, mixture in enumerate(mix_path[1:]):
        mixupdate_file = filecreation.mixupdate_hed(runset['models'], mixture)
        trainer.remix(
               os.path.join("models", runset['name'], "hmm8_%d" % mix_path[i]),
               os.path.join("models", runset['name'], "hmm1_%d" % mixture),
               mixupdate_file,
               runset['spmodellist']
               )
        for j in range(1,runs):
            trainer.reestimation(trainconfig,
                    spmlffile,
                    featurefile,
                    os.path.join("models", runset['name'], "hmm%d_%d" % (j, mixture)),
                    os.path.join("models", runset['name'], "hmm%d_%d" % ((j+1), mixture)),
                    runset['spmodellist'],
                    True, cores
                    )
    if not runset['multiset'] and runset['cleanup']:
        folder = os.path.join("features", runset['name'])
        logging.info("Deleting feature files for %s" % runset['name'])
        for root, dirs, files in os.walk(folder, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        os.rmdir(folder)
        logging.info("Deleted featurefiles for %s" % runset['name'])
