from executer import execute
import logging
import os
import util
import math

hcopy_bilin_path = "/home/studenter/bjornarg/bilinhtk/htk/bin.darwin/bin/HCopy"
hcopy_path = "/home/studenter/bjornarg/htk/bin.mac/bin/HCopy"
hcopy_mwf_path = "/home/studenter/bjornarg/htkmult/htk/bin/HCopy"

def feature_extraction(config, hcopyfilelist, cores=-1, bilin=False, mwf=False, mlf=None):
    if bilin:
        hcopy = hcopy_bilin_path
    elif mwf:
        hcopy = hcopy_mwf_path
    else:
        hcopy = hcopy_path
    hcopylist = [hcopy, '-C', config]
    if mlf is not None:
        hcopylist.extend(['-I', mlf])
    if cores > 0:
        files = split_file(hcopyfilelist, cores)
        processes = []
        for f in files:
            processes.append(execute(hcopylist+['-S', f], False))
        for p in processes:
            (stdout, stderr) = p.communicate()
            if stdout != "":
                logging.error("Command output: %s" % stdout)
            if stderr != "":
                logging.error("Command errors: %s" % stderr)
    else:
        execute(hcopylist+['-S', hcopyfilelist])

def init_proto(config, filelist, targetdir, protofile, mlf, f=0.01):
    util.make_dir(targetdir)
    execute(['HCompV', '-C', config, '-f', str(f), 
        '-I', mlf,
        '-m', '-S', filelist, '-M',
        targetdir, protofile])

def hard_init_proto(config, filelist, targetdir, protofile, mlf, model):
    util.make_dir(targetdir)
    execute(['HInit', '-C', config, '-A', '-l', model, '-o', model,
        '-M', targetdir,
        '-S', filelist,
        '-I', mlf,
        protofile])

def hard_reestimation(config, targetdir, sourcedir, filelist, mlf, model):
    execute(['HRest', '-S', filelist, 
        '-I', mlf,
        '-l', model,
        '-M', targetdir,
        os.path.join(sourcedir, model)])

def reestimation(config, mlf, filelist, sourcedir, targetdir, modellist, binary=False, split=-1):
    sourcemacros = os.path.join(sourcedir, "macros")
    sourcemmf = os.path.join(sourcedir, "hmmdefs")
    util.make_dir(targetdir)
    command = ['HERest',]
    if binary:
        command.append("-B")
    if split <= 0:
        command.extend(['-C', config, '-I', mlf, '-t', '250.0', '150.0',
            '1000.0', '-S', filelist, '-H', sourcemacros, '-H', sourcemmf,
            '-M', targetdir, modellist])
        execute(command)
    else:
        processes = []
        accfiles = []
        new_files = split_file(filelist, split)
        for i, file in enumerate(new_files):
            accfiles.append(os.path.join(targetdir, "HER%i.acc" % (i+1)))
            cmd = list(command)
            cmd.extend([
                '-C', config, '-I', mlf, '-t', '250.0', '150.0', '1000.0',
                '-p', str(i+1), '-S', file, '-H', sourcemacros, 
                '-H', sourcemmf, '-M', targetdir, modellist])
            processes.append(execute(cmd, False))
        for p in processes:
            (stdout, stderr) = p.communicate()
            if stdout != "":
                logging.error("Command output: %s" % stdout)
            if stderr != "":
                logging.error("Command errors: %s" % stderr)
        command.extend([
            '-C', config, '-t', '250.0', '150.0', '1000.0', 
            '-H', sourcemacros, '-H', sourcemmf, 
            '-p', '0', '-M', targetdir, modellist,
            ])
        command.extend(accfiles)
        execute(command)

def remix(source, target, hed_file, hmm_list):
    util.make_dir(target)
    execute(['HHEd', '-H', os.path.join(source, "macros"),
        '-H', os.path.join(source, "hmmdefs"),
        '-M', target,
        hed_file,
        hmm_list])

def split_file(file, splits):
    new_files = []
    lines = open(file, 'r').readlines()
    jump = int(math.ceil(len(lines)/float(splits)))
    for i in range(0,len(lines),jump):
        flines = lines[i:i+jump]
        f = os.path.join("tmp", "file%s" % i)
        new_files.append(f)
        fo = open(f, 'w')
        fo.writelines(flines)
    return new_files

